#ifndef __CAMERA_CALIBRATION_H
#define __CAMERA_CALIBRATION_H

#include "cv.h"
#include "highgui.h"
#include <stdio.h>
#include <ctype.h>

//VideoInput Library
#include "videoInput.h"
#include "CameraCalibration.h"

//My includes
#include <iostream>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <string>
#include <math.h>
#include <time.h>
#include <windows.h>


#include "global_vars.h"
using namespace std;

// Used to calibrate a set of image pixels to world coordinates
// Parameters are worldC -> world coordinates as input by the user in the command prompt
// and imageC -> images coordinates as found by the location of the calibration target
// Returns pointer to openCV matrix which is solution to sum of least squared solution
// For best fit for a simple camera model.

CvMat* generate_calib_matrix_simple_9(double worldC[18], double imageC[9][3])
{
	//U is the world coordinates in 2-space
	CvMat U = cvMat(18,1,CV_64FC1, worldC);
	
	// Convert from c++ array to openCV matrix
	for(int i = 0; i<18; i++)
	{
		cvmSet(&U, i,0,worldC[i]);
	}
	//K is image coords as measured by camera
	//calculate K matrix
	CvMat* K = 0;
	K = cvCreateMat(18,11,CV_64FC1);
		
	// Convert from c++ array to openCV matrix
	for(int i = 0, row=0; i<9; i++, row+=2)
	{
		double x = imageC[i][0];
		double y = imageC[i][1];
		double z = 0;

		double subU = cvmGet(&U, 2*i, 0);
		double subV = cvmGet(&U, 2*i+1, 0);
		cout<<"SUB: "<<subU<<" "<<subV<<endl;
 	
		cvmSet(K, row, 0,  x);
		cvmSet(K, row, 1,  y); 
		cvmSet(K, row, 2,  z);
		cvmSet(K, row, 3,  1);
		cvmSet(K, row, 4,  0);
		cvmSet(K, row, 5,  0);
		cvmSet(K, row, 6,  0);
		cvmSet(K, row, 7,  0);
		cvmSet(K, row, 8,  0);
		cvmSet(K, row, 9,  0);
		cvmSet(K, row, 10, 0);

		cvmSet(K, row + 1, 0,  0);
		cvmSet(K, row + 1, 1,  0);
		cvmSet(K, row + 1, 2,  0);
		cvmSet(K, row + 1, 3,  0);
		cvmSet(K, row + 1, 4,  x);
		cvmSet(K, row + 1, 5,  y); 
		cvmSet(K, row + 1, 6,  z);
		cvmSet(K, row + 1, 7,  1);
		cvmSet(K, row + 1, 8,  0);
		cvmSet(K, row + 1, 9,  0);
		cvmSet(K, row + 1, 10, 0);
	}


	// Print U matrix
	cout<<"************* U *************"<<endl;
	for(int i = 0; i<9; i++)
	{
		cout<<setw(10)<<cvmGet(&U,2*i,0)<<setw(10)<<cvmGet(&U,2*i+1,0)<<endl;
	}

/*
	ifstream fin("matrix.txt");
	for (int i = 0; i<18; i++){
		for (int j = 0; j<11; j++){
			double temp_num;
			fin>>temp_num;
			cvmSet(K,i,j,temp_num);
		}
	}
	fin.close();


	ofstream fout("Kmatrix.txt");
	for (int i = 0; i<18; i++){
		for (int j = 0; j<11; j++){
			fout<<setw(20)<<cvmGet(K,i,j);
		}
		fout<<endl;
	}
	fout.close();
*/

	// Print K matrix
	cout<<"************* K *************"<<endl;
	for(int i = 0; i<18; i++)
	{
		for(int j = 0; j<11; j++)
		{
			cout<<cvmGet(K, i, j)<<" ";
		}
		cout<<endl;
	}

	// Solve SSD M matrix and display
	cout<<"************* M *************"<<endl;
	CvMat* M = cvCreateMat(11,1,CV_64FC1);
	cvSolve(K, &U, M, CV_SVD);
	
	// Put this solution into a 3x4 matrix so we can multiply
	// for calibration
	CvMat* CamCalib = cvCreateMat(3,4,CV_64FC1);
	cvSetZero(CamCalib);
	cvmSet(CamCalib, 0, 0, cvmGet(M,0,0));
	cvmSet(CamCalib, 0, 1, cvmGet(M,1,0));
	cvmSet(CamCalib, 0, 2, cvmGet(M,2,0));
	cvmSet(CamCalib, 0, 3, cvmGet(M,3,0));
	cvmSet(CamCalib, 1, 0, cvmGet(M,4,0));
	cvmSet(CamCalib, 1, 1, cvmGet(M,5,0));
	cvmSet(CamCalib, 1, 2, cvmGet(M,6,0));
	cvmSet(CamCalib, 1, 3, cvmGet(M,7,0));
	cvmSet(CamCalib, 2, 0, cvmGet(M,8,0));
	cvmSet(CamCalib, 2, 1, cvmGet(M,9,0));
	cvmSet(CamCalib, 2, 2, cvmGet(M,10,0));
	cvmSet(CamCalib, 2, 3, 1);

	// Ouput M matrix
	for(int i = 0; i<3; i++)
	{
		for(int j = 0; j<4; j++)
		{
			cout<<setw(15)<<cvmGet(CamCalib,i,j);
		}
		cout<<endl;
	}
	return CamCalib;
}

CvMat* generate_calib_matrix_advanced_9(double worldC[18], double imageC[9][3])
{
	//U is the world coordinates in 2-space
	CvMat U = cvMat(18,1,CV_64FC1, worldC);
	
	// Convert from c++ array to openCV matrix
	for(int i = 0; i<18; i++)
	{
		cvmSet(&U, i,0,worldC[i]);
	}
	//K is image coords as measured by camera
	//calculate K matrix
	CvMat* K = 0;
	K = cvCreateMat(18,11,CV_64FC1);
		
	// Convert from c++ array to openCV matrix
	for(int i = 0, row=0; i<9; i++, row+=2)
	{
		double x = imageC[i][0];
		double y = imageC[i][1];
		double z = 0;

		double subU = cvmGet(&U, 2*i, 0);
		double subV = cvmGet(&U, 2*i+1, 0);
		cout<<"SUB: "<<subU<<" "<<subV<<endl;
 	
		cvmSet(K, row, 0,  x);
		cvmSet(K, row, 1,  y); 
		cvmSet(K, row, 2,  z);
		cvmSet(K, row, 3,  1);
		cvmSet(K, row, 4,  0);
		cvmSet(K, row, 5,  0);
		cvmSet(K, row, 6,  0);
		cvmSet(K, row, 7,  0);
		cvmSet(K, row, 8,  -subU*x);
		cvmSet(K, row, 9,  -subU*y);
		cvmSet(K, row, 10, -subU*z);

		cvmSet(K, row + 1, 0,  0);
		cvmSet(K, row + 1, 1,  0);
		cvmSet(K, row + 1, 2,  0);
		cvmSet(K, row + 1, 3,  0);
		cvmSet(K, row + 1, 4,  x);
		cvmSet(K, row + 1, 5,  y); 
		cvmSet(K, row + 1, 6,  z);
		cvmSet(K, row + 1, 7,  1);
		cvmSet(K, row + 1, 8,  -subV*x);
		cvmSet(K, row + 1, 9,  -subV*y);
		cvmSet(K, row + 1, 10, -subV*z);
	}


	// Print U matrix
	cout<<"************* U *************"<<endl;
	for(int i = 0; i<9; i++)
	{
		cout<<setw(10)<<cvmGet(&U,2*i,0)<<setw(10)<<cvmGet(&U,2*i+1,0)<<endl;
	}

/*
	ifstream fin("matrix.txt");
	for (int i = 0; i<18; i++){
		for (int j = 0; j<11; j++){
			double temp_num;
			fin>>temp_num;
			cvmSet(K,i,j,temp_num);
		}
	}
	fin.close();


	ofstream fout("Kmatrix.txt");
	for (int i = 0; i<18; i++){
		for (int j = 0; j<11; j++){
			fout<<setw(20)<<cvmGet(K,i,j);
		}
		fout<<endl;
	}
	fout.close();
*/

	// Print K matrix
	cout<<"************* K *************"<<endl;
	for(int i = 0; i<18; i++)
	{
		for(int j = 0; j<11; j++)
		{
			cout<<cvmGet(K, i, j)<<" ";
		}
		cout<<endl;
	}

	// Solve SSD M matrix and display
	cout<<"************* M *************"<<endl;
	CvMat* M = cvCreateMat(11,1,CV_64FC1);
	cvSolve(K, &U, M, CV_SVD);
	
	// Put this solution into a 3x4 matrix so we can multiply
	// for calibration
	CvMat* CamCalib = cvCreateMat(3,4,CV_64FC1);
	cvSetZero(CamCalib);
	cvmSet(CamCalib, 0, 0, cvmGet(M,0,0));
	cvmSet(CamCalib, 0, 1, cvmGet(M,1,0));
	cvmSet(CamCalib, 0, 2, cvmGet(M,2,0));
	cvmSet(CamCalib, 0, 3, cvmGet(M,3,0));
	cvmSet(CamCalib, 1, 0, cvmGet(M,4,0));
	cvmSet(CamCalib, 1, 1, cvmGet(M,5,0));
	cvmSet(CamCalib, 1, 2, cvmGet(M,6,0));
	cvmSet(CamCalib, 1, 3, cvmGet(M,7,0));
	cvmSet(CamCalib, 2, 0, cvmGet(M,8,0));
	cvmSet(CamCalib, 2, 1, cvmGet(M,9,0));
	cvmSet(CamCalib, 2, 2, cvmGet(M,10,0));
	cvmSet(CamCalib, 2, 3, 1);

	// Ouput M matrix
	for(int i = 0; i<3; i++)
	{
		for(int j = 0; j<4; j++)
		{
			cout<<setw(15)<<cvmGet(CamCalib,i,j);
		}
		cout<<endl;
	}
	return CamCalib;
}




#endif