#ifndef __GLOBAL_VARS_H
#define __GLOBAL_VARS_H

#include "XBeePackets.h"


//******************************************************//
//********ALL THE GLOBAL VARIABLES**********************//
//******************************************************//
#define PI 3.141592654
#define SEND_COUNTER_PERIOD 20	//The program will send position data out every SEND_COUNTER_PERIOD frames

// This number will be multiplied by max_dot_distance to determine 
// how close targets on different cameras can be before they are
// assumed to be the same target
#define BLIND_SPOT_REJECT_SCALER 1.5	

//Camera will discard targets this many pixels from the edge
//to prevent mistaken identities due to clipping
#define IGNORE_MARGIN 40


//define values to indicate if you want black dots on white background, or white dots on black background
#define BLACK_ON_WHITE 1
#define WHITE_ON_BLACK 2
#define DOT_COLOR BLACK_ON_WHITE //Choose if you want black dots on white background, or white dots on black background.

//time stamp
long int timestamp=0;

//variables for Xbee Packet
int returnCode=0;	//returns a code indicating success or failure
float rawData[DATA_ARRAY_LENGTH]={0};	//data to be sent out in a packet
int bytes_written=0;					//number of bytes written to serial port

// global images
IplImage *image = 0, *thresholded_image = 0, *grey = 0, *world_image = 0, *circle_image = 0;
IplImage *rawImage[4], *greyImage[4], *thresholdedImage[4], *thresholdedImageColor[4];

//for reading from VI (video input library)
unsigned char* new_image;	

// Camera capture numbers
int captureID[4];
int cameraID[4] = {-1, -1, -1, -1};  //maps camera to quadrant
int num_cams_ordered = 0;
bool camera_quadrants_matched = 0;
bool calibration_complete = 0;
bool recalibrate_cameras = 0;

// Capture Source Library
videoInput* VI;

// calibration matrices
CvMat* imageC = cvCreateMat(4,1,CV_64FC1);	
CvMat* worldC = cvCreateMat(3,1,CV_64FC1);	
CvMat* dotStorage = cvCreateMat(20,6,CV_64FC1);
CvMat* U, K, M;
CvMat* camCalib[4];		//calibration matrices
double world_cam1[18], world_cam2[18], world_cam3[18], world_cam4[18];
double world_center_x;
double world_center_y;

// real time parameters
double threshold = 70;	//black/white threshold
bool erode = 0;		//image erosion operations
double max_dot_distance = 65;	//maximum distance between dots
double area_desired = 0;		//theshold for dot sizes; reject anything smaller than this
int runMode = 0;				//not sure what this is

// mouse interrupt parameters
char* status = 0;
int flags = 0;

// file stream writers
ofstream fOut, fX, fY, fA;

// frame capture size
int fWidth = 640, fHeight = 480;

// gui paramters
IplImage *gui = 0;
const int thumbWidth = 640, thumbHeight=480;	//change this parameter if you want a smaller screen
const int guiWidth = thumbWidth*2, guiHeight = thumbHeight*2;	//the gui is two thumbnails high and two thumbnails wide

string message1 = "", message2 = "";
string mainWindow = "Indoor Localization System";
string vidWindow = "Capture Source";
	
	//initialize fps tracking vars
	// may want to change this to keep track of last 10 frames
	// so that the averaging is not so rough for longer times
	int numFrames = 0;
	long int startTime = 0;
	double fps;

//XBee Packet Handler
PacketHandler* packetHandlerPtr;

//******************************************************//
//********CUSTOM DATA STRUCTURES************************//
//******************************************************//
//for holding info about individual circles "dots"
struct dotData
{
	double ix;
	double iy;
	double area;
	double wx;
	double wy;
	double wz;
};

//for holding info about groups of dots called targets
struct target
{
	int numDots;		//Number of Dots
	dotData dots[9];	//Individual Dot Information
	dotData group;		//Joint information
	dotData minDot;

	double angle;
	double angleoffset;

	int x1, y1, x2, y2;
              
	int num_idd;
	double idd[36];
	double smallest_idd;

	int ID;
	double min_error;

	target *nxt;
};

// for holding info about "classifiers" - trained targets
struct classifier
{
	int ID;
	int numDots;
	double angleoffset;

	int num_idd;
	double idd[36];
	double smallest_idd;

	classifier *nxt;
};



#endif